package zero1hd.rhythmbullet.audio.visualizer;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Pixmap.Format;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.utils.Array;

import zero1hd.rhythmbullet.audio.MusicManager;

public class BasicVisualizer extends VisualizerCore {
	private Pixmap pixmap;
	private Texture barTexture;
	private int barWidth;
	private int binsPerBar;
	private int spaceBetweenBars;
	private Sprite[] bars;
	private float[] barHeights;
	private int smoothRange;
	private float barHeightMultiplier;
	private float rotation;
	private Vector2 angleRot;
	private boolean flip;
	private Array<MirrorVisualizer> mirrors;
	private boolean reverse;
	private float maxAvgHeight;
	private float currentAvg;
	
	public BasicVisualizer() {
		super(Gdx.graphics.getWidth(), Gdx.graphics.getHeight()/2, 0, 0);
		mirrors = new Array<>();
		pixmap = new Pixmap(2, 2, Format.RGBA8888);
		pixmap.setColor(Color.WHITE);
		pixmap.fill();
		barCount = 75;
		smoothRange = 2;
		angleRot = new Vector2(MathUtils.cosDeg(rotation), MathUtils.sinDeg(rotation));
		bars = new Sprite[barCount];
		barHeights = new float[barCount];
		barTexture = new Texture(pixmap);
		for (int i = 0; i < bars.length; i++) {
			bars[i] = new Sprite(barTexture);
		}
		
		updatePositionInfo();
		pixmap.dispose();
	}
	
	@Override
	public void render(Batch batch, float parentAlpha) {
		if (mm != null) {
			for (int i = 0; i < bars.length; i++) {
				bars[i].draw(batch);
				for (int j = 0; j < mirrors.size; j++) {
					mirrors.get(j).render(i, batch, parentAlpha, bars);
				}
			}
		}
		super.render(batch, parentAlpha);
	}
	
	public void modify(float delta) {
		//Averaging bins together
		for (int i = 0; i < barCount; i++) {
			float barHeight = 0;
			for (int j = 0; j < binsPerBar; j++) {
				barHeight += Math.abs(audioPCM[j+i*binsPerBar]);
			}
			barHeight /= binsPerBar;
			barHeight *= barHeightMultiplier;
			barHeights[i] = barHeight;
		}
		currentAvg = 0;
		for (int i = 0; i < barCount; i++) {
			int avg = 0;
			//Averaging the bars
			for (int range = 0; range < smoothRange; range++) {
				if (i+range < barCount) {
					avg += barHeights[i+range];
				}
				if (i-range >= 0) {
					avg += barHeights[i-range];
				}
			}
			avg /= smoothRange*2;
			barHeights[i] = avg;
			currentAvg += barHeights[i];
			if (bars[i].getHeight() > barHeights[i]) {
				bars[i].setSize(barWidth, bars[i].getHeight() - (20f*delta*(bars[i].getHeight()-barHeights[i])));
			} else {
				bars[i].setSize(barWidth, bars[i].getHeight() + (27f*delta*(barHeights[i] - bars[i].getHeight())));
			}
		}
		currentAvg /= barHeights.length;
		if (currentAvg > maxAvgHeight) {
			maxAvgHeight = currentAvg;
		}
	}
	
	@Override
	public void setMM(MusicManager mm) {
		super.setMM(mm);
		maxAvgHeight = 0;
		currentAvg = 0;
		float validBins = (5500/((mm.getSampleRate()/2)/((audioPCM.length/2)+1)));
		Gdx.app.debug("Visualizer", "valid frequency bins " + validBins);
		binsPerBar = MathUtils.round((validBins/barCount));
		barHeights = new float[barCount];
	}
	
	@Override
	public void dispose() {
		barTexture.dispose();
		if (mm != null) {
			mm.dispose();
		}
		super.dispose();
	}
	
	public void updatePositionInfo() {
		barHeightMultiplier = height*0.032f;
		int barSpace = 0;
		angleRot.set(MathUtils.cosDeg(rotation), MathUtils.sinDeg(rotation));
		
		barWidth = MathUtils.round((float) width/(float) barCount);
		barWidth -= spaceBetweenBars;
		for (int i = 0; i < bars.length; i++) {
			barSpace = i*(barWidth+spaceBetweenBars);
			if (flip) {
				bars[i].setRotation(rotation+180);
			} else {
				bars[i].setRotation(rotation);
			}
			if (reverse) {
				bars[bars.length-i-1].setPosition(xPos + barSpace*angleRot.x, yPos + barSpace*angleRot.y);
			} else {
				bars[i].setPosition(xPos + barSpace*angleRot.x, yPos + barSpace*angleRot.y);
			}
			for (int mirrorIndex = 0; mirrorIndex < mirrors.size; mirrorIndex++) {
				mirrors.get(mirrorIndex).position(i, barWidth, spaceBetweenBars);
			}
		}

	}
	
	public float getActualWidth() {
		return (barWidth+spaceBetweenBars)*(barCount - 1);
	}
	
	public void setColor(Color color) {
		for (int i = 0; i < bars.length; i++) {
			bars[i].setColor(color);
		}
		for (int i = 0; i < mirrors.size; i++) {
			mirrors.get(i).setColor(color);
		}
	}
	
	public void setColor(float r, float g, float b, float a) {
		for (int i = 0; i < bars.length; i++) {
			bars[i].setColor(r, g, b, a);
		}
		for (int i = 0; i < mirrors.size; i++) {
			mirrors.get(i).setColor(r, g, b, a);
		}
	}
	
	public void setRotation(float rotation) {
		this.rotation = rotation;
	}
	
	public void flip() {
		flip = flip ? false : true;
	}
	
	public boolean isFlipped() {
		return flip;
	}
	
	public void addMirrorVisualizer(MirrorVisualizer mirror) {
		mirror.setup(bars, xPos, yPos, rotation);
		mirrors.add(mirror);
		updatePositionInfo();
	}
	
	public void removeMirrorVisualizer(MirrorVisualizer mirror) {
		mirrors.removeValue(mirror, true);
	}
	
	public void setSpaceBetweenBars(int spaceBetweenBars) {
		this.spaceBetweenBars = spaceBetweenBars;
	}
	
	public int getBarWidth() {
		return barWidth;
	}
	
	public void reverse() {
		reverse = reverse ? false : true;
	}
	
	public boolean isReversed() {
		return reverse;
	}
	
	public Sprite[] getBars() {
		return bars;
	}
	
	public float getCurrentAvg() {
		return currentAvg;
	}
	
	public float getMaxAvgHeight() {
		return maxAvgHeight;
	}
	
	public int getSpaceBetweenBars() {
		System.out.println(spaceBetweenBars);
		return spaceBetweenBars;
	}
}
